#! /usr/bin/env python
'''
Module Name:    Utils.py

Description:    Contains helper functions 

Copyright:      Toshiba Confidential

                OCO Source Materials

                Copyright (C) 2012,2013 Toshiba Global Commerce Solutions, Inc.

                The source code for this program is not published
                or otherwise divested of its trade secrets,
                irrespective of what has been deposited with the
                U. S. Copyright Office.

Initial Author: Francisco Liwa

Function:

Notes:

Dependencies:    None

Restrictions:    None

Last Modified: 30 September 2013

'''
import os
import sys
import subprocess
import time

#importing helper modules 
from monitorupgrade import TLog as TLOG

#Globals
FILE_FOUND = 1
FILE_NOT_FOUND = 0
EXIT_SUCCESS = 0
EXIT_FAILURE = 1 
UNKNOWN = -1
NO_MONITORS_ATTACHED = 0

#Error Codes
ERROR_UPDATE_FAILED = 3
ERROR_CODE_DEVICE_NOT_EXIST = 4
ERROR_CODE_OPERATION_FAILED = 11
ERROR_CODE_INVALID_PARAM = 21

#Error Strings
ERROR_STR_DEVICE_NOT_EXIST = 'Device not exist'
ERROR_STR_OPERATION_FAILED = 'Operation failed'
ERROR_STR_INVALID_PARAM = 'Invalid Parameter'
ERROR_STR_Runtime = 'Runtime error'

MUS_LOG_FILE_DIR = '/var/log/toshiba/mus'

#sampo stuffs here
SAMPO_UTIL = '/opt/toshiba/mus/op_utils/sampo/4820MonUtil'
RECOVERY_FILE_SUSE = '/etc/init.d/after.local'
RECOVERY_STR = '4820MonUtil'

#ELO Stuffs here    
ELO_DOWNLOAD = '/opt/toshiba/mus/op_utils/elo/EloDownload.sh'
ELO_CONFIG = '/opt/toshiba/mus/op_utils/elo/EloConfig.sh'
ELO_RECOVERY_FILE = '/opt/toshiba/mus/op_utils/elo/recovery.txt'
BOOT_COUNTER ='/tmp/reboot_count'
#==========================================================================================================
'''
doesFileExist - Returns True if the file exist, False otherwise
'''
#==========================================================================================================

def doesFileExist(filename):
    
    if os.path.exists(filename) and os.path.isfile(filename) and os.access(filename, os.R_OK):
        #print filename + ' exists and is readable'
        TLOG.logger.info('doesFileExist()-Found! :'+ filename)
        return True
    else:
        #print filename  + ' does not exist'
        TLOG.logger.info('doesFileExist()-Not Found! :'+ filename)
        #raise IOError
        return False
#==========================================================================================================
'''
runShell - run any shell command
'''
#==========================================================================================================
def runShell(cmd):
    
    #TLOG.logger.info(cmd)    
    process = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE)
    while True:
        line = process.stdout.readline()
        #print line
        if not line:
            break
        TLOG.logger.info(line)    
        process.wait()
    
    return process.returncode



#==========================================================================================================
'''
runSubprocess - This will call the subprocess module to run the command in the cmd string
'''
#==========================================================================================================
def runSubprocess(cmd,proc,cwd_=None,time_out=60):
    
    TLOG.logger.info( str(cmd) )
    #std_err = None
    #try:
    process = subprocess.Popen(cmd,shell=True, stdout=subprocess.PIPE,stderr=subprocess.PIPE, cwd = cwd_ )        
    
#    if (proc == 'ELODOWNLOAD-4820_2Lx5Lx'):
#        t = 0
#        while t < time_out and process.poll() is None:
#            time.sleep(1)
#            t += 1
#                 
#        #process.wait()
#        if process.poll() is None:
#            process.terminate()
#            ret = -1
#        else:
#            ret = process.returncode
#    else:    
    while True:
        line = process.stdout.readline()
        #print line
        if not line:
            break
        line = line.strip('\n')
        TLOG.logger.info('        Subprocess[' + proc + '] OUTPUT-->' + ''  +  line )
        msg = ''    
        if (proc == 'SampoVideoFlashTask_4820_21x51x' or proc == 'SampoVideoFlashTask_4820_2Lx5Lx'):
            if "DATA" in line:
                idx = line.find(">")
                for c in line[idx + 1:len(line)]:
                    if c == '<':
                        break
                    msg += c
                print msg + '\r'  
    
    process.wait()
    ret = process.returncode
    
    try:
        if ret == 139 and proc == 'ELO_RECOVER':               # TODO: ELO Recovery Success but with Segmentation Fault
            return EXIT_SUCCESS
        elif ret != EXIT_SUCCESS:
                  
            TLOG.logger.error('return code:' + str(ret) )
            
            #when the monitor is unplugged during Firmware update.
            if ( ret == ERROR_CODE_DEVICE_NOT_EXIST  or ret == ERROR_UPDATE_FAILED ) :  
                print '[TaskMgr::runSubprocess()] Error:' + ERROR_STR_Runtime
                raise RuntimeError('Device not exist')
            
            elif (ret == ERROR_CODE_OPERATION_FAILED ) :
                print '[TaskMgr::runSubprocess()] Error:' + ERROR_STR_OPERATION_FAILED
            
            elif (ret == ERROR_CODE_INVALID_PARAM ) :
                print '[TaskMgr::runSubprocess()] Error:' + ERROR_STR_INVALID_PARAM
                    
            elif (proc != 'ELOCONFIGUTILITY'):
                print '[TaskMgr::runSubprocess()] Error:' + str(ret)
                raise IOError(ret)
            #else:
                #raise IOError(ret)
        else:
            
            return EXIT_SUCCESS
    
    except (ValueError):
        
        extype, value = sys.exc_info()[:2]
        print 'Value Error'     
        print >> sys.stderr, "Invalid argument in Popen"
        TLOG.logger.error("%s (%s)" % (extype, value) )
        # return 2 for "bad command line option"
        sys.exit(2)
    
    except (IOError):
        
        extype, value = sys.exc_info()[:2]
        print 'IO Error'     
        #print >> sys.stderr, "Error in process: %s" % proc 
        TLOG.logger.error("TaskMgr::runSubprocess()-[Error in process: %s] %s (s%)" % ( proc,repr(extype), repr(value) ) )
 
        cleanUp(False)
        
        return EXIT_FAILURE
        
    except (RuntimeError):      
       
        value = sys.exc_info()[:2]
        #print value
        print 'Runtime Error'
        #print >> sys.stderr, "TaskMgr::runSubprocess() - Error in process: %s" % proc 
        TLOG.logger.error("TaskMgr::runSubprocess()-[RuntimeError in process: [%s] [%s]" % ( proc, repr(value) ) )
        
        #if ( proc == 'SampoVideoFlashTask_4820_2Lx5Lx' or proc == 'SampoVideoFlashTask_4820_21x51x' ):
            #cleanUp(True)       # Sets the recovery value to True
        #elif (proc == 'ELODOWNLOAD-4820_2Lx5Lx' or proc == 'ELODOWNLOAD-4820_21x51x') :
            #cleanUp(False)
            
            
        return EXIT_FAILURE   
        
            
    except (OSError):
        
        extype, value = sys.exc_info()[:2]
        print 'OS Error'     
        #print >> sys.stderr, "ERROR: %s (%s)" % (extype, value)
        TLOG.logger.error("%s (%s)" % (extype, value) )
        # return 2 for "bad command line option"
        sys.exit(2)
    
    except:
        # all other exceptions get caught here
        extype, value = sys.exc_info()[:2]
        print 'CatchAll Error'
        #print >> sys.stderr, "ERROR: %s (%s)" % (extype, repr(value) )
        TLOG.logger.error("%s (%s)" % (extype,  repr(value) ) )
        sys.exit(1)
        
#==========================================================================================================
# cleanUp() -  delete firmware files from the fw_temp directory which MUS checks
#           -  delete updatask.conf configuration file
#==========================================================================================================        
def cleanUp(in_recovery):
       
        #cmd = []
        
        TLOG.logger.info('cleanUp()------------------START---------------')    
        
        if (not in_recovery):
            
        
            if doesFileExist(ELO_RECOVERY_FILE) == True:
                cmd1 = 'rm' + ' ' + ELO_RECOVERY_FILE
                #cmd.append(cmd1)
                TLOG.logger.info(cmd1)
            
                print cmd1
                rem_proc = runShell(cmd1)
         
             
           
        else:
            
            if doesFileExist(ELO_RECOVERY_FILE) == True:
                cmd1 = 'rm' + ' ' + ELO_RECOVERY_FILE
                #cmd.append(cmd1)
                TLOG.logger.info(cmd1)
            
                print cmd1
                rem_proc = runShell(cmd1)
                if rem_proc == 0:
                    runShell('reboot')
                    
        TLOG.logger.info('cleanUp()-------------------END----------------\n')     
        
def doEloRecover(fw_path):
        
        TLOG.logger.info('doEloRecover() -----------------START---------------')
        eloRecoverCmd = 'sh ' +  ELO_DOWNLOAD + '  -select=fix  -firmware_file=' + str(fw_path)
        print eloRecoverCmd
        if doesFileExist(ELO_DOWNLOAD) != True:
            print 'EloAnakFlashTask::run()-No ELO Download script found!'
            TLOG.logger.error('Elo4820_21x51xFlashTask::run()-No ELO Download script found!')
            raise IOError('No ELO Download script found!')
        else:
            ret = runSubprocess(eloRecoverCmd,'ELO_RECOVER')
             
        if ret != EXIT_SUCCESS:
            return ret #return immediately
        else:
            print 'doEloRecover() - Task Completed! - Please check log file at '  + MUS_LOG_FILE_DIR
            TLOG.logger.info('doEloRecover() -----------------END-----------------\n')
            return ret

def updateBootCount():
    
        current_count=0
        if doesFileExist(BOOT_COUNTER) != True:
            f = open(BOOT_COUNTER,'w+')
            f.write('1')
            f.close()
        else:
            f = open(BOOT_COUNTER,'r+')
            lines = f.readlines()
            for line in lines:
                if not line:
                    break
                if line.isdigit() == True:
                    current_count = int(line.strip('\n'))
            
            #updating the counter
            f.seek(0)           # beginning of the file
            f.truncate()        # empty the file
            
            current_count += 1  # increment by 1
            
            f.write( str(current_count) )
            f.close()

def getBootCount():
        current_count=0
        if doesFileExist(BOOT_COUNTER) == True:
            f = open(BOOT_COUNTER,'r+')
            lines = f.readlines()
            for line in lines:
                if not line:
                    break
                if line.isdigit() == True:
                    current_count = int(line.strip('\n'))
                    
        return current_count