#!/usr/bin/env python
'''
Module Name:    op_install.py

Description:    This is a helper script during op installations
Copyright:      Toshiba Confidential

                OCO Source Materials

                Copyright (C) 2012,2013 Toshiba Global Commerce Solutions, Inc.

                The source code for this program is not published
                or otherwise divested of its trade secrets,
                irrespective of what has been deposited with the
                U. S. Copyright Office.

Initial Author: Francisco Liwa

Function:    

Notes:

Dependencies:    None

Restrictions:    None

Last Modified: 25 September 2013

'''

import os,sys,subprocess
import logging
import datetime

VER = 3
OP_LOG_FILE_DIR = '/var/log/toshiba/mus/op_installation'
LOG_NAME = 'mus_op_1.1-'+ str(VER)
#OP_NAME = 'toshiba-mus_op-suse-1.1-'+VER
UP_RPMS_DIR = '/opt/toshiba/mus/RpmUpdates'


    

#Create the logger object
logger = logging.getLogger(LOG_NAME)
logger.setLevel(logging.INFO)  

# make some handlers
date_now = datetime.datetime.now()
logname = LOG_NAME + '-' + str(date_now) + '.log'

name = os.path.join(OP_LOG_FILE_DIR,logname)
#print 'Logging to file '+ name
console_handler = logging.StreamHandler() # by default, sys.stderr
file_handler = logging.FileHandler(name)

# set logging levels
console_handler.setLevel(logging.WARNING)
file_handler.setLevel(logging.INFO)

# create formatter
formatter = logging.Formatter('%(asctime)s - %(name)s:[%(levelname)s] - %(message)s')

# add formatter to file_handler
file_handler.setFormatter(formatter)
console_handler.setFormatter(formatter)
  
# add handlers to logger
logger.addHandler(console_handler)
logger.addHandler(file_handler)      


#===============================================================================
# #method to run any shell commands from python
#===============================================================================
def runCmd(cmd):
    
    logger.info(cmd)   
    process = subprocess.Popen(cmd,shell=True, stdout=subprocess.PIPE)
    while True:
        line = process.stdout.readline()
        if not line:
            break
        line = line.strip("\n")
        logger.info(line)    
        #print line
    


#===============================================================================
# #Basically , just create the file system at the start and then
# #Register the Monitor Upgrade Services
#===============================================================================

print '\nStarting Monitor Upgrade Service installation...\n'
logger.info('Starting installation...')

print 'Checking dependencies...'
logger.info('Checking dependencies...')

#print 'lsmod | grep elo'
runCmd ('lsmod | grep elo')


print ('Extracting  ELO TOOLs...') 
logger.info('Extracting  ELO TOOLs...')
        
#print 'Extracting /opt/toshiba/MonitorUpgrade/op/4820/ELO/elo.tar...'
runCmd ('tar -C /opt/toshiba/mus/op_utils/elo/ -xvf  /opt/toshiba/mus/op_utils/elo/elo.tar')

#moving elo binaries to /usr/bin/

runCmd ('mv  /opt/toshiba/mus/op_utils/elo/elo_download /opt/toshiba/mus/op_utils/elo/elo_config /usr/bin/')
runCmd ('mv  /opt/toshiba/mus/op_utils/elo/FwTouchUpgradeUtil /usr/bin/')
runCmd('chmod +x /usr/bin/elo_download /usr/bin/elo_config /usr/bin/FwTouchUpgradeUtil')
#runCmd ('rm -rf  /opt/toshiba/mus/op_utils/elo/elo.tar')


print ('Extracting  SAMPO TOOLs...') 
logger.info('Extracting  SAMPO TOOLs...')

runCmd ('tar -C /opt/toshiba/mus/op_utils/sampo -xvf /opt/toshiba/mus/op_utils/sampo/sampo.tar')
#runCmd ('mv  /opt/toshiba/mus/op_utils/sampo/4820MonUtil /usr/bin/')
#runCmd ('rm -rf /opt/toshiba/mus/op_utils/sampo/sampo.tar')

print ('Extracting  MUS HLPRS TOOLs...') 
logger.info('Extracting  MUS_HLPRS TOOLs...')

runCmd ('tar -C /opt/toshiba/mus/op_utils/hlprs -xvf /opt/toshiba/mus/op_utils/hlprs/mus_hlprs.tar')
runCmd ('mv  /opt/toshiba/mus/op_utils/hlprs/rstelo  /usr/bin/')
runCmd ('mv  /opt/toshiba/mus/op_utils/hlprs/lstmon  /usr/bin/')
runCmd ('mv  /opt/toshiba/mus/op_utils/hlprs/chk_elo_patch  /usr/bin/')
runCmd ('mv  /opt/toshiba/mus/op_utils/hlprs/chk_fousb  /usr/bin/')

print ('Extracting  License files...')
logger.info('Extracting License files...')
runCmd ('tar -C /opt/toshiba/mus/LA/ -xvf /opt/toshiba/mus/LA/LA.tar')

runCmd ('rm -rf /opt/toshiba/mus/LA/LA.tar')

print 'Creating MUS directory...'
logger.info('Creating MUS directory...')

logger.info('Checking directory -/opt/toshiba/mus')

#===============================================================================
# Creating directory for UP RPMS
# This is where we place all UP RPMS
#===============================================================================
if os.path.exists('/opt/toshiba/mus/'):
        
        #print '/opt/toshiba/MonitorUpgrade/mus/scripts/  exist!'
        logger.info('/opt/toshiba/mus/ exist!')
       
        print ('Creating UP RPMS directory...') 
        logger.info('Creating UP RPMS directory')
        
        if  os.path.exists(UP_RPMS_DIR) != True :
            runCmd ('mkdir '  + UP_RPMS_DIR)
        

#===============================================================================
# Creating UnInstall script
#===============================================================================
#if os.path.exists('/opt/toshiba/mus/'):
#        
#        #print '/opt/toshiba/MonitorUpgrade/mus/scripts/  exist!'
#        logger.info('Creating UnInstall script')
#       
#        print ('Creating UnInstall script') 
#                
#        runCmd ('tar -xvf opt/toshiba/mus/mus_op_unistall.tar' )
#    
#===============================================================================
# #Create a copy of our service to /etc/init.d`
# #This will enable our script to be invoke during boot up of the machine
# #Note: MUST make sure the copy in the /etc/init.d should not have the extension 
# ".py" or else this won't work
#===============================================================================
#print 'Adding init script to /etc/init.d/...'
logger.info('Adding init script to /etc/init.d/...')

#print 'Copying MonitorUpgradeSvc...'
runCmd('cp -p /usr/local/lib/python2.6/site-packages/initscript/MonitorUpgradeSvc.py /etc/init.d/MonitorUpgradeSvc')


#===============================================================================
# #Now Register the service using chkconfig
# #Opted to use this method instead of insserv,
# #They pretty much do the same function
#===============================================================================

runCmd('chmod +x /etc/init.d/MonitorUpgradeSvc')
#runCmd('chown root:root /etc/init.d/MonitorUpgradeSvc')

print 'Registering MonitorUpgradeSvc...'
logger.info('Registering MonitorUpgradeSvc...')

runCmd('chkconfig -a MonitorUpgradeSvc')
runCmd('chkconfig MonitorUpgradeSvc on')


#===============================================================================
# #DONE!
#===============================================================================
print '\nInstallation Complete!\n'
logger.info('Installation Complete!')
